clear all; clc; close all;

load '../data_input/T2012IncomeExclGainTop01.txt';
load '../data_input/T2012IncomeExclGainTop1.txt';
load ../data_input/T2012XIncome.txt
year = T2012XIncome;

etaincexcl1 = 1+log10(T2012IncomeExclGainTop01./T2012IncomeExclGainTop1);

etaincexcl1(year<1910) = [];
year(year<1910) = [];

sig2H = (0.15)^2;
sig2L = (0.1)^2;
muL = 0;
muH = 0.05;
muHp = muH + 0.08;
phi = 1/6;
death = 1/25;
the = 0.3; %fraction of people born in high growth regime
J = 2;

I= 2000;
xmin = -2;
xmax = 15;
x = linspace(xmin,xmax,I)';
dx = (xmax-xmin)/(I-1);
dx2 = dx^2;

[x0 i_entry] = min(abs(x));

Aswitch = [-speye(I)*phi,speye(I)*phi;sparse(I,I),sparse(I,I)];

sL = muL*ones(I,1);
sH = muH*ones(I,1);

%CONSTRUCT MATRIX
X_L = -min(sL,0)/dx + sig2L/(2*dx2);
Y_L = -max(sL,0)/dx + min(sL,0)/dx - sig2L/dx2;
Z_L =  max(sL,0)/dx + sig2L/(2*dx2);

X_H = -min(sH,0)/dx + sig2H/(2*dx2);
Y_H = -max(sH,0)/dx + min(sH,0)/dx - sig2H/dx2;
Z_H =  max(sH,0)/dx + sig2H/(2*dx2);

A_L=spdiags(Y_L,0,I,I)+spdiags(X_L(2:I),-1,I,I)+spdiags([0;Z_L(1:I-1)],1,I,I);
A_L(I,I) = Y_L(I) + Z_L(I);
A_L(1,1) = X_L(1)+ Y_L(1);

A_H=spdiags(Y_H,0,I,I)+spdiags(X_H(2:I),-1,I,I)+spdiags([0;Z_H(1:I-1)],1,I,I);
A_H(I,I) = Y_H(I) + Z_H(I);
A_H(1,1) = X_H(1)+ Y_H(1);

A = [A_H,sparse(I,I);sparse(I,I),A_L] + Aswitch;


%ADJUST FOR DEATH
A = A - death*speye(I*J);
A(:,i_entry) = A(:,i_entry) + death*the;
A(:,I+i_entry) = A(:,I+i_entry) + death*(1-the);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION FOR INITIAL DISTRIBUTION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I*J,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I*J-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
ff = AT2\b;
ff_sum = ff'*ones(I*J,1)*dx;
ff = ff./ff_sum;

p(:,1) = ff(1:I); p(:,2) = ff(I+1:2*I);

f = p(:,1) + p(:,2);


%initial distribution
f0  = f;
ff0  = ff;

for i=1:I
    F(i) = sum(f(1:i)*dx);
end

perc = 0.99;
[val index] = min(abs(F - perc));
for i=1:I-1
    PL(i) = -(log(f(i+1,1))-log(f(i,1)))/dx;
end
PL1 = PL(index);


%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%
muH = muHp;

sL = muL*ones(I,1);
sH = muH*ones(I,1);

%CONSTRUCT MATRIX
X_L = -min(sL,0)/dx + sig2L/(2*dx2);
Y_L = -max(sL,0)/dx + min(sL,0)/dx - sig2L/dx2;
Z_L =  max(sL,0)/dx + sig2L/(2*dx2);

X_H = -min(sH,0)/dx + sig2H/(2*dx2);
Y_H = -max(sH,0)/dx + min(sH,0)/dx - sig2H/dx2;
Z_H =  max(sH,0)/dx + sig2H/(2*dx2);

A_L=spdiags(Y_L,0,I,I)+spdiags(X_L(2:I),-1,I,I)+spdiags([0;Z_L(1:I-1)],1,I,I);
A_L(I,I) = Y_L(I) + Z_L(I);
A_L(1,1) = X_L(1)+ Y_L(1);

A_H=spdiags(Y_H,0,I,I)+spdiags(X_H(2:I),-1,I,I)+spdiags([0;Z_H(1:I-1)],1,I,I);
A_H(I,I) = Y_H(I) + Z_H(I);
A_H(1,1) = X_H(1)+ Y_H(1);

A = [A_H,sparse(I,I);sparse(I,I),A_L] + Aswitch;


%ADJUST FOR DEATH
A = A - death*speye(I*J);
A(:,i_entry) = A(:,i_entry) + death*the;
A(:,I+i_entry) = A(:,I+i_entry) + death*(1-the);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I*J,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I*J-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
ff = AT2\b;
ff_sum = ff'*ones(I*J,1)*dx;
ff = ff./ff_sum;

p(:,1) = ff(1:I); p(:,2) = ff(I+1:2*I);
f = p(:,1) + p(:,2);

for i=1:I
    F(i) = sum(f(1:i)*dx);
end

perc = 0.99;
[val index] = min(abs(F - perc));
for i=1:I-1
    PL(i) = -(log(f(i+1))-log(f(i)))/dx;
end
PL1 = PL(index);


%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%

T = 100;
N = T*10;
dt = T/N;

ff_t(:,1) = ff0;
for n=1:N
   ff_t(:,n+1) = (speye(I*J,I*J) - dt*AT)\ff_t(:,n);
   ff_t_sum(n) =sum(ff_t(:,n)*dx);
   p(:,1) = ff_t(1:I,n); p(:,2) = ff_t(I+1:2*I,n);
   f_t(:,n) = p(:,1) + p(:,2);
end


p = 0.99;

for n=1:N
    for i=1:I
        F_t(i,n) = sum(f_t(1:i,n)*dx);
    end
    [val index] = min(abs(F_t(:,n) - p));
    PL1_t(n) = -(log(f_t(index+1,n))-log(f_t(index,n)))/dx;
end

time = (1:N)*dt;
time = time + 1973;
long_time = 1900:2050;
NN = length(long_time);

h1 = plot(year,etaincexcl1,'b',time,1./PL1_t,'g',long_time,ones(NN,1)/PL1,'r--','LineWidth',2)
set(gca,'FontSize',16)
legend(h1,'Data (Piketty and Saez)','Model w High Growth Regime','Model Steady State','Location','SouthEast')
xlabel('Year')
ylabel('\eta(1)')
xlim([1950 2050])
ylim([0.35 0.7])
set(findobj('color','g'),'Color',[0 0.5 0]);
print -depsc ../figures/fig5b.eps
