%NEEDS INPUT FROM fig4a.m
clear all; clc; close all;

load ../data_input/T2012WageTop1.txt
load ../data_input/T2012XWage.txt
year = T2012XWage;
top1_data = T2012WageTop1;

top1_data(year<1910) = [];
year(year<1910) = [];

eta0 = 0.33;
alpha = 1/eta0;
sig = 0.1;
sig2 = sig^2;
death = 1/30;
mu = death/alpha -alpha*(sig2/2);

sigp = sqrt(0.025);
sig2p = sigp^2;

I = 1000;
i_entry = round(I/4);
xmin = -2;
xmax = 8;
x = linspace(xmin,xmax,I)';
dx = (xmax-xmin)/(I-1);
dx2 = dx^2;

s = mu*ones(I,1);

%CONSTRUCT MATRIX
X = -min(s,0)/dx + sig2/(2*dx2);
Y = -max(s,0)/dx + min(s,0)/dx - sig2/dx2;
Z =  max(s,0)/dx + sig2/(2*dx2);

A=spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
A(I,I) = Y(I) + Z(I);
A(1,1) = X(1)+ Y(1);

%ADJUST FOR DEATH
A = A - death*speye(I);
A(:,i_entry) = A(:,i_entry) + death;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION FOR INITIAL DISTRIBUTION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
f = AT2\b;
f_sum = f'*ones(I,1)*dx;
f = f./f_sum;

plot(x,s,x,zeros(1,I),'--')

%initial distribution
f0  = f;

for i=1:I
    F(i) = sum(f(1:i)*dx);
end

p = 0.99;
[val index] = min(abs(F - p));
integrand = exp(x).*f.*dx;
total = sum(integrand);
top1 = sum(integrand(index:I))/total*100;

%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%
sig = sigp;
sig2 = sig^2;

s = mu*ones(I,1);

%CONSTRUCT MATRIX
X = -min(s,0)/dx + sig2/(2*dx2);
Y = -max(s,0)/dx + min(s,0)/dx - sig2/dx2;
Z =  max(s,0)/dx + sig2/(2*dx2);

A=spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
A(I,I) = Y(I) + Z(I);
A(1,1) = X(1)+ Y(1);

%ADJUST FOR DEATH
A = A - death*speye(I);
A(:,i_entry) = A(:,i_entry) + death;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
f = AT2\b;
f_sum = f'*ones(I,1)*dx;
f = f./f_sum;

for i=1:I
    F(i) = sum(f(1:i)*dx);
end

p = 0.99;
[val index] = min(abs(F - p));
integrand = exp(x).*f.*dx;
total = sum(integrand);
top1 = sum(integrand(index:I))/total*100;


%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%

T = 100;
N = T*10;
dt = T/N;

f_t(:,1) = f0;
for n=1:N
   f_t(:,n+1) = (speye(I,I) - dt*AT)\f_t(:,n);
   f_t_sum(n) =sum(f_t(:,n)*dx);
end

p = 0.99;

for n=1:N
    for i=1:I
        F_t(i,n) = sum(f_t(1:i,n)*dx);
    end
    [val index] = min(abs(F_t(:,n) - p));
    integrand = exp(x).*f_t(:,n).*dx;
    total = sum(integrand);
    top1_t(n) = sum(integrand(index:I))/total*100;
end

time = (1:N)*dt;

time = time + 1970; %start in 1970 so as to give model best shot
time_baseline = time + 3; %start baseline in 1973 as before
long_time = 1900:2050;
NN = length(long_time);

load top1_t.mat %generated by fig4a.m

h1 = plot(year,top1_data,'b',year_baseline,top1_data_baseline,'b--',time,top1_t,'g',time_baseline,top1_t_baseline,'g--',long_time,ones(NN,1)*top1,'r--','LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Wage Data','Income Data','Model, wage calibration','Model, income calibration','Steady State, wage cal.','Location','NorthWest')
xlabel('Year')
ylabel('Top 1% Income Share')
xlim([1950 2050])
ylim([5 19])
set(findobj('color','g'),'Color',[0 0.5 0]);
print -depsc ../figures/fig6a.eps
