%NEEDS INPUT FROM fig4b.m
clear all; clc; close all;

load ../data_input/T2012WageTop01.txt;
load ../data_input/T2012WageTop1.txt;
load ../data_input/T2012XWage.txt
year = T2012XWage;

etawag1 = 1+log10(T2012WageTop01./T2012WageTop1);

etawag1(year<1910) = [];
year(year<1910) = [];

eta0 = 0.315;
alpha = 1/eta0;
sig = 0.1;
sig2 = sig^2;
death = 1/30;
mu = death/alpha -alpha*(sig2/2);

sigp = sqrt(0.025);

I= 1000;
i_entry = round(I/4);
xmin = -2;
xmax = 8;
x = linspace(xmin,xmax,I)';
dx = (xmax-xmin)/(I-1);
dx2 = dx^2;

s = mu*ones(I,1);

%CONSTRUCT MATRIX
X = -min(s,0)/dx + sig2/(2*dx2);
Y = -max(s,0)/dx + min(s,0)/dx - sig2/dx2;
Z =  max(s,0)/dx + sig2/(2*dx2);

A=spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
A(I,I) = Y(I) + Z(I);
A(1,1) = X(1)+ Y(1);

%ADJUST FOR DEATH
A = A - death*speye(I);
A(:,i_entry) = A(:,i_entry) + death;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION FOR INITIAL DISTRIBUTION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
f = AT2\b;
f_sum = f'*ones(I,1)*dx;
f = f./f_sum;

plot(x,s,x,zeros(1,I),'--')

%initial distribution
f0  = f;

for i=1:I
    F(i) = sum(f(1:i)*dx);
end

p = 0.99;
[val index] = min(abs(F - p));
for i=1:I-1
    PL(i) = -(log(f(i+1))-log(f(i)))/dx;
end
PL1 = PL(index);

%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%
sig = sigp;
sig2 = sig^2;

s = mu*ones(I,1);

%CONSTRUCT MATRIX
X = -min(s,0)/dx + sig2/(2*dx2);
Y = -max(s,0)/dx + min(s,0)/dx - sig2/dx2;
Z =  max(s,0)/dx + sig2/(2*dx2);

A=spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
A(I,I) = Y(I) + Z(I);
A(1,1) = X(1)+ Y(1);

%ADJUST FOR DEATH
A = A - death*speye(I);
A(:,i_entry) = A(:,i_entry) + death;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
f = AT2\b;
f_sum = f'*ones(I,1)*dx;
f = f./f_sum;

for i=1:I
    F(i) = sum(f(1:i)*dx);
end

p = 0.99;
[val index] = min(abs(F - p));
for i=1:I-1
    PL(i) = -(log(f(i+1))-log(f(i)))/dx;
end
PL1 = PL(index);


%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%

T = 100;
N = T*10;
dt = T/N;

f_t(:,1) = f0;
for n=1:N
   f_t(:,n+1) = (speye(I,I) - dt*AT)\f_t(:,n);
   f_t_sum(n) =sum(f_t(:,n)*dx);
end

p = 0.99;

for n=1:N
    for i=1:I
        F_t(i,n) = sum(f_t(1:i,n)*dx);
    end
    [val index] = min(abs(F_t(:,n) - p));
    PL1_t(n) = -(log(f_t(index+1,n))-log(f_t(index,n)))/dx;
end

time = (1:N)*dt;
time = time + 1970; %start in 1970 so as to give model best shot
time_baseline = time + 3; %start baseline in 1973 as before
long_time = 1900:2050;
NN = length(long_time);

load PL1_t.mat %generated by fig4b.m

h1 = plot(year,etawag1,'b',year_baseline,etaincexcl1,'b--',time,1./PL1_t,'g',time_baseline,1./PL1_t_baseline,'g--',long_time,ones(NN,1)/PL1,'r--','LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Wage Data','Income Data','Model, wage calibration','Model, income calibration','Steady State, wage cal.','Location','SouthEast')
xlabel('Year')
ylabel('\eta(1)')
xlim([1950 2050])
ylim([0.3 0.65])
set(findobj('color','g'),'Color',[0 0.5 0]);
print -depsc ../figures/fig6b.eps
