clear all; clc; close all;

%USE PIKETTY-ZUCMAN RETURN ESTIMATES, AUERBACH-HASSETT TAX SERIES, AND
%GROWTH RATE FROM PENN WORLD TABLES (PWT)

%Returns from Piketty-Zucman
filename = '../data_input/Piketty_Zucman_USA_countryfile.xls';
PZ_data = xlsread(filename);
PZ_year = PZ_data(:,1);
PZ_r = PZ_data(:,2);

i65 = find(PZ_year == 1965);
i09 = find(PZ_year == 2009);
PZ_r([1:i65-1,i09+1:end])=[];
PZ_year([1:i65-1,i09+1:end])=[];

%Tax rates from Auerbach-Hassett/NBER
filename = '../data_input/Auerbach_Hassett_tax_data.xls';
AH_data = xlsread(filename);
AH_year = AH_data(:,1);
AH_tax = AH_data(:,2);

i65 = find(AH_year == 1965);
i09 = find(AH_year == 2009);
AH_tax([1:i65-1,i09+1:end])=[];

% GROWTH RATE g from PWT
% DOWNLOAD MATLAB VERSION OF PWT from http://www.rug.nl/research/ggdc/data/pwt/v80/pwt80_mat.zip 
% and place in directory
load PWT.mat
usa = find(ismember(PWT.country,'United States'));  % find index of USA
% compute US's GDP/capita (using national account data)
income = PWT.rgdpna(usa,:);
income_per_capita = PWT.rgdpna(usa,:)./PWT.pop(usa,:);

% compute growth rates
g_raw = diff(log(income_per_capita));

date = datevec(PWT.year);
PWT_year = date(:,1);

g = smooth(g_raw,20);
i65 = find(PWT_year == 1965);
i09 = find(PWT_year == 2009);
g([1:i65-1,i09+1:end])=[];

%COMPUTE AFTER_TAX RATE OF RETURN
r_g_data = (1-AH_tax).*PZ_r-g;

plot(PZ_year,r_g_data,'b','LineWidth',2)
set(gca,'Fontsize',16)
ylabel('r-g')
xlabel('Year')
xlim([1965 2010])
print -depsc ../figures/fig8a.eps

%generate file for fig8b.m: r-g series with many time grid points
i70 = find(PZ_year == 1970);
i08 = find(PZ_year == 2008);

T = 100;
N = T*10;
dt = T/N;
time=(1:N)*dt;

T1 = 2008 - 1970+1;
N1 = T1*10;
r_g = zeros(N,1);
r_g(1:N1) = kron(r_g_data(i70:i08),ones(10,1));
r_g(N1:N)=r_g_data(i08);

save r_g r_g
