%NEEDS INPUT FROM fig8a.m (the r-g series)
clear all; clc; close all;

load '../data_input/WealthTop1_SZ.txt';
load '../data_input/XWealth_SZ.txt';
load '../data_input/WealthTop1_SCF_Wolff.txt';
load '../data_input/XWealth_SCF_Wolff.txt';

year = XWealth_SCF_Wolff;
year_SZ = XWealth_SZ;

top1_data = WealthTop1_SCF_Wolff*100;
top1_SZ = WealthTop1_SZ*100;

eta0 = 0.675;
alpha = 1/eta0;
sig = 0.35;
sig2 = sig^2;
mu = -alpha*(sig2/2);

T = 100;
N = T*10;
dt = T/N;
time=(1:N)*dt;

load r_g.mat %generated by fig8a.m

MPC = r_g(1) - sig2/2 -mu; %mu = r - g - MPC - sig2/2;
mu_t = r_g - MPC - sig2/2;

I= 1000;
xmin = -2;
xmax = 8;
x = linspace(xmin,xmax,I)';
dx = (xmax-xmin)/(I-1);
dx2 = dx^2;

w = 0.03;
s = w*exp(-x) + mu;

%CONSTRUCT MATRIX
X = -min(s,0)/dx + sig2/(2*dx2);
Y = -max(s,0)/dx + min(s,0)/dx - sig2/dx2;
Z =  max(s,0)/dx + sig2/(2*dx2);

A=spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
A(I,I) = Y(I) + Z(I);
A(1,1) = X(1)+ Y(1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION FOR INITIAL DISTRIBUTION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
f = AT2\b;
f_sum = f'*ones(I,1)*dx;
f = f./f_sum;

plot(x,s,x,zeros(1,I),'--')

%initial distribution
f0  = f;

for i=1:I
    F(i) = sum(f(1:i)*dx);
end

p = 0.99;
[val index] = min(abs(F - p));

%WEALTH SHARES
integrand = exp(x).*f.*dx;
total = sum(integrand);
top1 = sum(integrand(index:I))/total*100;

%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%
%trigerred by an increase in r-g

mu = mu_t(end);

s = w*exp(-x) + mu;

%CONSTRUCT MATRIX
X = -min(s,0)/dx + sig2/(2*dx2);
Y = -max(s,0)/dx + min(s,0)/dx - sig2/dx2;
Z =  max(s,0)/dx + sig2/(2*dx2);

A=spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
A(I,I) = Y(I) + Z(I);
A(1,1) = X(1)+ Y(1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATIONARY FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I-i_fix)];
AT2 = AT;
AT2(i_fix,:) = row;

%Solve linear system
f = AT2\b;
f_sum = f'*ones(I,1)*dx;
f = f./f_sum;


for i=1:I
    F(i) = sum(f(1:i)*dx);
end

p = 0.99;
[val index] = min(abs(F - p));

%WEALTH SHARES
integrand = exp(x).*f.*dx;
total = sum(integrand);
top1 = sum(integrand(index:I))/total*100;

%%%%%%%%%%%%%%
% TRANSITION %
%%%%%%%%%%%%%%

f_t(:,1) = f0;
for n=1:N
    s = w*exp(-x) + mu_t(n);
    %CONSTRUCT MATRIX
    X = -min(s,0)/dx + sig2/(2*dx2);
    Y = -max(s,0)/dx + min(s,0)/dx - sig2/dx2;
    Z =  max(s,0)/dx + sig2/(2*dx2);
    A=spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
    A(I,I) = Y(I) + Z(I);
    A(1,1) = X(1)+ Y(1);
    AT = A';
    f_t(:,n+1) = (speye(I,I) - dt*AT)\f_t(:,n);
    f_t_sum(n) =sum(f_t(:,n)*dx);
end

p = 0.99;

for n=1:N
for i=1:I
    F_t(i,n) = sum(f_t(1:i,n)*dx);
end
    [val index] = min(abs(F_t(:,n) - p));
    integrand = exp(x).*f_t(:,n).*dx;
    total = sum(integrand);
    top1_t(n) = sum(integrand(index:I))/total*100;
end

time = (1:N)*dt;
plot(time,top1_t)

time = time + 1970;
long_time = 1900:2050;
NN = length(long_time);

h1 = plot(year,top1_data,'b',year_SZ,top1_SZ,'b--',time,top1_t,'g','LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Data (SCF)','Data (Saez-Zucman)','Model Transition','Location','NorthWest')
xlabel('Year')
ylabel('Top 1% Wealth Share')
xlim([1950 2030])
set(findobj('color','g'),'Color',[0 0.5 0]);
print -depsc ../figures/fig8b.eps
